import React from "react";
import { Toaster } from "@/components/ui/toaster";
import { Toaster as Sonner } from "@/components/ui/sonner";
import { TooltipProvider } from "@/components/ui/tooltip";
import { QueryClient, QueryClientProvider } from "@tanstack/react-query";
import { BrowserRouter, Routes, Route, Navigate } from "react-router-dom";
import { HelmetProvider } from 'react-helmet-async';
import { lazy, Suspense } from 'react';

// Lazy load all page components to reduce initial bundle size
const Index = lazy(() => import("./pages/Index"));
const ConversionPage = lazy(() => import("./pages/ConversionPage"));
const AllConversions = lazy(() => import("./pages/AllConversions"));
const AboutPage = lazy(() => import("./pages/AboutPage"));
const UnitsGuidePage = lazy(() => import("./pages/UnitsGuidePage"));
const NotFound = lazy(() => import("./pages/NotFound"));
const TermsPage = lazy(() => import("./pages/TermsPage"));
const PrivacyPolicy = lazy(() => import("./pages/PrivacyPolicy"));
const SitemapPage = lazy(() => import("./pages/SitemapPage"));
const SquareFootageCalculator = lazy(() => import("./pages/SquareFootageCalculator"));
const FeetToInches = lazy(() => import("./pages/FeetToInches"));
const FeetToMeters = lazy(() => import("./pages/FeetToMeters"));
const FeetToMiles = lazy(() => import("./pages/FeetToMiles"));
const CentimeterToInches = lazy(() => import("./pages/CentimeterToInches"));
const CentimeterToFeet = lazy(() => import("./pages/CentimeterToFeet"));
const CentimeterToMeter = lazy(() => import("./pages/CentimeterToMeter"));
const MeterToCentimeter = lazy(() => import("./pages/MeterToCentimeter"));
// New landing pages
const LandUnitsPage = lazy(() => import("./pages/LandUnitsPage"));
const LengthUnitsPage = lazy(() => import("./pages/LengthUnitsPage"));
const CalculatorsPage = lazy(() => import("./pages/CalculatorsPage"));
const GuidesPage = lazy(() => import("./pages/GuidesPage"));
const BighaToAcreBlog = lazy(() => import("./pages/BighaToAcreBlog"));
const Infinity = React.lazy(() => import("./pages/Infinity"));

// Lazy load utilities - only load when needed
const ScrollToTop = lazy(() => import("./components/ScrollToTop"));
const RedirectHandler = lazy(() => import("./components/RedirectHandler"));

import { seoRoutes, getAlternativeUrls, SEORoute } from "./config/seoRoutes.config";

// Loading component for better UX during code splitting
const PageLoadingSpinner = () => (
  <div className="min-h-screen flex items-center justify-center bg-gradient-to-br from-blue-50 to-white">
    <div className="flex flex-col items-center space-y-4">
      <div className="animate-spin rounded-full h-12 w-12 border-b-2 border-blue-600"></div>
      <p className="text-gray-600 text-sm">Loading...</p>
    </div>
  </div>
);

const queryClient = new QueryClient();

// Get alternative URLs for redirect handling
const alternativeUrls = getAlternativeUrls();

const App = () => (
  <HelmetProvider>
    <QueryClientProvider client={queryClient}>
      <TooltipProvider>
        <Toaster />
        <Sonner />
        <BrowserRouter>
          <Suspense fallback={<PageLoadingSpinner />}>
            <ScrollToTop />
            <RedirectHandler />
            <Routes>
              <Route path="/" element={<Index />} />
              
              {/* Legacy redirect routes for /convert/ paths */}
              <Route path="/convert/*" element={<RedirectHandler />} />
            
            {/* SEO-optimized canonical routes */}
            {Object.entries(seoRoutes).map(([path, config]: [string, SEORoute]) => {
              // Route specific components based on conversion type
              if (config.fromUnit === 'feet' && config.toUnit === 'inches') {
                return <Route key={path} path={path} element={<FeetToInches />} />;
              }
              if (config.fromUnit === 'inches' && config.toUnit === 'feet') {
                return <Route key={path} path={path} element={<FeetToInches initialFromUnit="inch" initialToUnit="foot" />} />;
              }
              if (config.fromUnit === 'feet' && config.toUnit === 'meters') {
                return <Route key={path} path={path} element={<FeetToMeters />} />;
              }
              if (config.fromUnit === 'meters' && config.toUnit === 'feet') {
                return <Route key={path} path={path} element={<FeetToMeters initialFromUnit="meter" initialToUnit="foot" />} />;
              }
              if (config.fromUnit === 'feet' && config.toUnit === 'miles') {
                return <Route key={path} path={path} element={<FeetToMiles />} />;
              }
              if (config.fromUnit === 'miles' && config.toUnit === 'feet') {
                return <Route key={path} path={path} element={<FeetToMiles initialFromUnit="mile" initialToUnit="foot" />} />;
              }
              if (config.fromUnit === 'centimeter' && config.toUnit === 'inches') {
                return <Route key={path} path={path} element={<CentimeterToInches />} />;
              }
              if (config.fromUnit === 'inches' && config.toUnit === 'centimeter') {
                return <Route key={path} path={path} element={<CentimeterToInches initialFromUnit="inch" initialToUnit="centimeter" />} />;
              }
              if (config.fromUnit === 'centimeter' && config.toUnit === 'feet') {
                return <Route key={path} path={path} element={<CentimeterToFeet />} />;
              }
              if (config.fromUnit === 'feet' && config.toUnit === 'centimeter') {
                return <Route key={path} path={path} element={<CentimeterToFeet initialFromUnit="foot" initialToUnit="centimeter" />} />;
              }
              if (config.fromUnit === 'centimeter' && config.toUnit === 'meter') {
                return <Route key={path} path={path} element={<CentimeterToMeter />} />;
              }
              if (config.fromUnit === 'meter' && config.toUnit === 'centimeter') {
                return <Route key={path} path={path} element={<MeterToCentimeter />} />;
              }
              if (path === '/square-footage-calculator') {
                return <Route key={path} path={path} element={<SquareFootageCalculator />} />;
              }
              
              // Default to ConversionPage for other conversions
              return <Route key={path} path={path} element={<ConversionPage />} />;
            })}
            
            {/* Alternative URL redirects to canonical URLs */}
            {alternativeUrls.map(({ alternative, canonical }) => (
              <Route 
                key={alternative}
                path={alternative}
                element={<Navigate to={canonical} replace />}
              />
            ))}
            
            {/* Main conversion route - fallback for dynamic conversions */}
            <Route path="/:conversionPath" element={<ConversionPage />} />
            
            {/* Category landing pages */}
            <Route path="/land" element={<LandUnitsPage />} />
            <Route path="/length" element={<LengthUnitsPage />} />
            <Route path="/calculators" element={<CalculatorsPage />} />
            <Route path="/guides" element={<GuidesPage />} />

            {/* Indian Land Measurement Guide */}
            <Route path="/guides/indian-land-measurement-guide" element={React.createElement(lazy(() => import("./pages/guides/indian-land-measurement-guide")))} />
            
            {/* Main site pages */}
            <Route path="/all-land-units-conversions" element={<AllConversions />} />
            
            {/* Static pages */}
            <Route path="/about" element={<AboutPage />} />
            <Route path="/units-guide" element={<UnitsGuidePage />} />
            <Route path="/terms" element={<TermsPage />} />
            <Route path="/privacy" element={<PrivacyPolicy />} />
            <Route path="/sitemap" element={<SitemapPage />} />
            
            {/* Blog/Guide pages */}
            <Route path="/bigha-to-acre-guide" element={<BighaToAcreBlog />} />
            
            {/* Secret pages */}
            {/* Secret pages - lazy loaded */}
            <Route path="/infinity" element={
              <Suspense fallback={<PageLoadingSpinner />}>
                <Infinity />
              </Suspense>
            } />
            
            {/* Not found page for anything else */}
            <Route path="*" element={<NotFound />} />
          </Routes>
          </Suspense>
        </BrowserRouter>
      </TooltipProvider>
    </QueryClientProvider>
  </HelmetProvider>
);

export default App;
